#!/bin/bash

#Preupgrade Assistant performs system upgradability assessment
#and gathers information required for successful operating system upgrade.
#Copyright (C) 2013 Red Hat Inc.
#Petr Stodulka <pstodulk@redhat.com>
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
. /usr/share/preupgrade/common.sh
COMPONENT="distribution"
#END GENERATED SECTION

check_root
fix_script="fix_SELinuxCustomPolicy.sh"

rm -f solution.txt
echo "Custom SELinux policy modules couldn't be found by sesearch. This is fixed by removing selinux module sandbox.pp which is repalced by sandboxX.pp and is disabled by default on RHEL7 systems. 

This solve some other issues between sandbox.pp and sandboxX.pp too. So module
is removed be default now by postcript: $POSTUPGRADE_DIR/$fix_script" > solution.txt
/bin/cp $fix_script $POSTUPGRADE_DIR/$fix_script

exit $RESULT_FIXED

