#!/bin/bash


#Preupgrade Assistant performs system upgradability assessment
#and gathers information required for successful operating system upgrade.
#Copyright (C) 2013 Red Hat Inc.
#Petr Hracek <phracek@redhat.com>
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
. /usr/share/preupgrade/common.sh
check_applies_to "aide"
COMPONENT="aide"
#END GENERATED SECTION
COMPONENT="aide"
AIDE_LOG="/var/log/aide/aide.log"
FOUND=0

# Aide installed but never run
if [[ ! -f $AIDE_LOG ]]; then
    log_info "AIDE was installed but never run."
    exit $RESULT_INFORMATIONAL
fi

log_medium_risk "AIDE is being used for 'guarding' the system integrity."
exit $RESULT_FAIL
